#!/usr/bin/env python
#
#    Copyright (C) 2005 Grigory Bakunov
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
''' Simple script what can help newbies to configure Leonardo '''

import os, os.path, sys, pwd

class SConfig:
    """ Primitive wrapper to leonardo config file.
    It's very ugly cause I want to save all file formating, justification,
    identation etc.
    """
    def __init__(self, file = '../cgi-bin/leonardo_config.py'):
        ''' Constructor. '''
        self.file = file
        self.lines = []
        self.udots = {}
        self.load()
        self.parse()

    def load(self):
        ''' Load file content to array of lines '''
        if os.path.isfile(self.file):
            _fl = open(self.file)
            self.lines = _fl.readlines()
            _fl.close()
        else:
            raise SystemError('File %s not found, exiting' % self.file)

    def parse(self):
        ''' Parse lines array to dictionary with config key as a key.'''
        _lineNum = 0
        for line in self.lines:
            if len(line) > 0 and line.find('=') > 0 and line[0]!= '#':
                [dot,value] = line.strip().split('=')
                self.udots[dot.strip()] = ((dot, value), value.strip()[1:-1], _lineNum)
            _lineNum += 1

    def change(self, key, value):
        ''' Change one key in dictionary and change appropriate part of config array. '''
        if self.udots.has_key(key):
            ((dot, exvalue), was, lineNum) = self.udots[key]
            self.lines[lineNum] = '%s= "%s" # (was %s)\n' % (dot, value, exvalue)

    def userGet(self, key, comment, default):
        ''' Get user input and put it to config array '''
        inpString = '%s (default: %s)\n>>> ' % (comment, default)
        input = raw_input(inpString)
        if len(input) == 0:
            print '... using default'
            input = default
        self.change(key, input.strip())
        return input.strip()

    def yesNo(self, comment, default = 'n'):
        if default == 'n':
            defaultAns = 'no'
        else:
            defaultAns = 'yes'
        inpString = '%s (default is %s)\n>>> ' % (comment, defaultAns)
        input = raw_input(inpString)
        if len(input) == 0:
            print '... using default'
            input = default
        if input in ['n', 'no', 'NO', 'N', 'No']:
            return False
        elif input in ['y', 'yes', 'YES', 'Y', 'Yes']:
            return True
        print 'Agrr... What do you mean?'
        return self.yesNo(comment, default)
        
    def shotout(self):
        ''' Show ready to use config file '''
        for line in self.lines:
            sys.stdout.write(line)

    def shotOutToFile(self):
        ''' Write config array to appropriate file '''
        fp = open(self.file, 'w')
        for line in self.lines:
            fp.write(line)
        fp.close()
            

if __name__ == "__main__":
    # Absolute path to leonardo/bin directory
    mainDir = os.path.abspath('..')
    # Get real name from passwd file
    try:
        login = os.getlogin()
        (pwnam, none, none, none, name, home, none) = pwd.getpwnam(login)
    except:
        # Not in unix? Too bad
        login = 'anonymous'
        name  = 'I Am'
        home  = mainDir
    try:
        sconfig = SConfig()
    except:
        print 'Config file not found. Try to move to Leonardo bin directory.'
        sys.exit(0)
    try:
        print '[[ Technical details ]]'
        sconfig.userGet('data_dir', 'Main data directory (finished with "/" sign)', os.path.join(mainDir, 'data') + '/')
        sconfig.userGet('site_url', 'Site main URL', 'http://localhost:8000/')

        print '[[ Site information ]]'
        sconfig.userGet('site_title', 'Your site name', name)
        sconfig.userGet('site_sub_title', 'Your site subtitle', 'My little site')
        sconfig.userGet('blog_title', 'Your weblog title', 'My Blog')
        sconfig.userGet('blog_author', 'Blog Author', name)
        sconfig.userGet('copyright_holder', 'Copyright holder', name)
    except:
        print '\n<<< Interrupted by user >>>'
        sys.exit(0)
    sconfig.shotOutToFile()
    print '<<< Site Configuration Saved >>>'
    print 'Now open file %s with your editor and changepath to' % \
          os.path.join(mainDir, 'cgi-bin', 'leonardo.py')
    print 'Leonardo libraries. I think what "%s" will be good enough.' % \
          os.path.join(mainDir, 'lib')
    print 'Have a fun with Leonardo.'
